;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'hypermedia '("htmlize"))
(custom-add-loads 'hexl '("hexl"))
(custom-add-loads 'languages '("crontab" "winmgr-mode" "xrdb-mode"))
(custom-add-loads 'matching '("whitespace-mode"))
(custom-add-loads 'processes '("ansi-color" "flyspell"))
(custom-add-loads 'flyspell '("flyspell"))
(custom-add-loads 'winmgr '("winmgr-mode"))
(custom-add-loads 'xrdb '("xrdb-mode"))
(custom-add-loads 'ansi-colors '("ansi-color"))
(custom-add-loads 'htmlize '("htmlize"))
(custom-add-loads 'whitespace '("whitespace-mode"))
(custom-add-loads 'fill '("filladapt"))
(custom-add-loads 'data '("hexl"))
(custom-add-loads 'outlines '("whitespace-mode"))
(custom-add-loads 'folding '("folding"))
(custom-add-loads 'wp '("scribe"))
(custom-add-loads 'scribe '("scribe"))
(custom-add-loads 'filladapt '("filladapt"))
(custom-add-loads 'tools '("autoinsert" "folding"))
(custom-add-loads 'auto-insert '("autoinsert"))
(custom-add-loads 'crontab '("crontab"))

;;; custom-load.el ends here
